@@ -329,9 +329,15 @@ char *lockname (char *tmp,char *fname)
 {
   char *s = strrchr (fname,'/');
   struct stat sbuf;
+#ifdef __linux__
   if (stat (fname,&sbuf))	/* get file status */
-    sprintf (tmp,"/tmp/.%s",s ? s : fname);
+    sprintf (tmp,"/var/lock/.%s",s ? s : fname);
+  else sprintf (tmp,"/var/lock/.%hx.%lx",sbuf.st_dev,sbuf.st_ino);
+#else
+  if (stat (fname,&sbuf))	/* get file status */
+    sprintf (tmp,"/tmp/.%s",s ? s : fname); 
   else sprintf (tmp,"/tmp/.%hx.%lx",sbuf.st_dev,sbuf.st_ino);
+#endif /* __linux__ */
   return tmp;			/* note name for later */
 }
 
