@@ -1184,7 +1184,11 @@ int mtx_lock (int fd,char *lock,int op)
 				/* get data for this file */
   if (fstat (fd,&sbuf)) return -1;
 				/* make temporary file name */
+#ifdef __linux__
+  sprintf (lock,"/var/lock/.%hx.%lx",sbuf.st_dev,sbuf.st_ino);
+#else
   sprintf (lock,"/tmp/.%hx.%lx",sbuf.st_dev,sbuf.st_ino);
+#endif /* __linux__ */
   if ((ld = open (lock,O_RDWR|O_CREAT,
 		  (int) mail_parameters (NIL,GET_LOCKPROTECTION,NIL))) < 0)
     return NIL;
