@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: composer.c,v 4.38 1994/08/19 00:25:25 mikes Exp $";
+static char rcsid[] = "$Id: composer.c,v 4.45 1994/09/29 22:29:00 mikes Exp $";
 #endif
 /*
  * Program:	Pine composer routines
@@ -206,6 +206,7 @@ PICO  *mp;
      */
     ods.p_off  = 0;
     ods.p_line = COMPOSER_TOP_LINE;
+    ods.top_l = ods.cur_l = NULL;
 
     headents = mp->headents;
     /*--- initialize the fields in the headerent structure ----*/
@@ -600,8 +601,9 @@ int f, n;
 	    { char fn[NLINE], sz[32], cmt[NLINE];
 
 	      if(AskAttach(fn, sz, cmt)){
-		  char buf[NLINE];
-		  int  a_e;
+		  int	 a_e;
+		  struct hdr_line *lp;
+
                   /*--- Find headerentry that is attachments (only first) --*/
                   for(a_e = 0; headents[a_e].name != NULL; a_e++ )
                     if(headents[a_e].is_attach){
@@ -611,21 +613,43 @@ int f, n;
 			break;
 		    }
 
-		  /* update display */
-		  sprintf(buf, "%s (%s) \"%s\"%s", fn, sz, cmt, 
-			  (headents[a_e].hd_text->text[0]=='\0') ? "":",");
+		  /* append new attachment line */
+		  for(lp = headents[a_e].hd_text; lp->next; lp=lp->next)
+		    ;
 
-		  if(FormatLines(headents[a_e].hd_text, buf,
-				 term.t_ncol - headents[a_e].prlen,
-				 headents[a_e].break_on_comma, 0) == -1){
-		      emlwrite("\007Format lines failed!", NULL);
+		  /* build new attachment line */
+		  if(lp->text[0]){		/* adding a line? */
+		      strcat(lp->text, ",");	/* append delimiter */
+		      if(lp->next = HALLOC()){	/* allocate new line */
+			  lp->next->prev = lp;
+			  lp->next->next = NULL;
+			  lp = lp->next;
+		      }
+		      else{
+			  emlwrite(
+				 "\007Can't allocate line for new attachment!",
+				 NULL);
+			  break;
+		      }
 		  }
 
-		  if(SyncAttach() < 0)
-		    emlwrite("\007Problem attaching: %s", fn);
+		  sprintf(lp->text, "%s (%s) \"%.*s\"", fn, sz, 80, cmt);
+
+		  /* validate the new attachment, and reformat if needed */
+		  if(status = SyncAttach()){
+		      if(status < 0)
+			emlwrite("\007Problem attaching: %s", fn);
+
+		      if(FormatLines(headents[a_e].hd_text, "",
+				     term.t_ncol - headents[a_e].prlen,
+				     headents[a_e].break_on_comma, 0) == -1){
+			  emlwrite("\007Format lines failed!", NULL);
+			  break;
+		      }
+		  }
 
 		  UpdateHeader();
-		  PaintHeader(COMPOSER_TOP_LINE, FALSE);
+		  PaintHeader(COMPOSER_TOP_LINE, status != 0);
 		  PaintBody(1);
 	      }
 
@@ -672,7 +696,8 @@ int f, n;
 		char dir[NLINE], fn[NLINE], sz[NLINE];
 
 		strcpy(dir, gmode&MDCURDIR ? "." : gethomedir(NULL));
-		if(FileBrowse(dir, fn, sz) == 1){ /* got a new file */
+		fn[0] = '\0';
+		if(FileBrowse(dir, fn, sz, FB_READ) == 1){ /* got a new file */
 		    char buf[NLINE];
 		    sprintf(buf, "%s%c%s (%s) \"\"%s", dir, C_FILESEP, fn, sz, 
 			    (!headents[ods.cur_e].hd_text->text[0]) ? "":",");
@@ -806,9 +831,9 @@ header_downline(beyond)
     }
 
     if(new_l){					/* fixup new pointers	    */
+	ods.cur_l = (ods.cur_e != new_e) ? headents[new_e].hd_text : new_l;
 	ods.cur_e = new_e;
-	ods.cur_l = new_l;
-	if(ods.p_off > (len = strlen(new_l->text)))
+	if(ods.p_off > (len = strlen(ods.cur_l->text)))
 	  ods.p_off = len;
     }
 
@@ -1076,6 +1101,7 @@ int	allowedit;
 		}
 		strng = ods.cur_l->text;
 		ods.p_len = strlen(strng);
+		headents[ods.cur_e].sticky = 1;
 		continue;
 
 	      case (CTRL|'U') :			/* un-delete deleted lines */
@@ -1087,6 +1113,7 @@ int	allowedit;
 		    ods.p_off = 0;		/* dot hasn't moved! */
 		    strng = ods.cur_l->text;
 		    ods.p_len = strlen(strng);
+		    headents[ods.cur_e].sticky = 1;
 		}
 		else
 		  emlwrite("Problem Unkilling text", NULL);
@@ -1141,6 +1168,7 @@ int	allowedit;
 		    continue;
 		}
 
+		headents[ods.cur_e].sticky = 1;
 		pputc(strng[ods.p_off++], 0); 	/* drop through and rubout */
 
 	      case 0x7f       :			/* blast previous char */
@@ -1209,6 +1237,8 @@ int	allowedit;
 
 		movecursor(ods.p_line, ods.p_off+headents[ods.cur_e].prlen);
 
+		headents[ods.cur_e].sticky = 1;
+
 		if(skipmove)
 		  continue;
 
@@ -2176,12 +2206,14 @@ struct headerentry *entry;
 	    InitEntryText(s, entry);		/* arrange new one */
 	    zotentry(line); 			/* blast old list o'entries */
 	}
+
 	if(fcc && !entry->affected_entry->sticky){
 	    line = entry->affected_entry->hd_text;
-	    if(strcmp(line->text, fcc)) /* make sure they see it if changed */
+	    if(strcmp(line->text, fcc)){ /* make sure they see it if changed */
 		entry->affected_entry->display_it = 1;
-	    InitEntryText(fcc, entry->affected_entry);
-	    zotentry(line); 			/* blast old list o'entries */
+		InitEntryText(fcc, entry->affected_entry);
+		zotentry(line);			/* blast old list o'entries */
+	    }
 	}
 
         retval = TRUE;
@@ -2238,16 +2270,16 @@ strqchr(s, ch, q)
     int	 quoted = (q) ? *q : 0;
 
     for(; s && *s; s++){
-	if(*s == '"')
-	  quoted = !quoted;
+	if(*s == '"'){
+	    quoted = !quoted;
+	    if(q)
+	      *q = quoted;
+	}
 
 	if(!quoted && *s == ch)
 	  return(s);
     }
 
-    if(q)
-      *q = quoted;
-
     return(NULL);
 }
 
