@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: file.c,v 4.10 1994/07/15 23:32:52 mikes Exp $";
+static char rcsid[] = "$Id: file.c,v 4.14 1994/10/11 23:46:51 mikes Exp $";
 #endif
 /*
  * Program:	High level file input and output routines
@@ -161,7 +161,8 @@ int f, n;
 		    else
 		      strcpy(dir, gmode&MDCURDIR ? "." : gethomedir(NULL));
 
-		    if((s = FileBrowse(dir, fname, NULL)) == 1){
+		    fname[0] = '\0';
+		    if((s = FileBrowse(dir, fname, NULL, FB_READ)) == 1){
 			if(gmode&MDSCUR){
 			    emlwrite("Can't insert file in restricted mode",
 				     NULL);
@@ -347,14 +348,32 @@ int f, n;
 	      case TRUE:
 		break;
 	      case (CTRL|'T'):
-		if(*fname && isdir(fname, NULL)){
-		    strcpy(shows, fname);
-		    fname[0] = '\0';
+		/* If we have a file name, break up into path and file name.*/
+		*shows = 0;
+		if(*fname) {
+		    if (isdir (fname, NULL)) {
+			/* fname is a directory. */
+			strcpy (shows, fname);
+			*fname = '\0';
+		    }
+		    else {
+			/* Find right most seperator. */
+			bufp = strrchr (fname, C_FILESEP);
+			if (bufp != NULL) {
+			    /* Copy directory part to 'shows', and file
+			     * name part to front of 'fname'. */
+			    *bufp = '\0';
+			    strcpy (shows, fname);
+			    memcpy (fname, bufp+1, strlen (bufp+1) + 1);
+			}
+		    }
 		}
-		else
+
+		/* If we did not end up with a valid directory, use home. */
+		if (!isdir (shows, NULL))
 		  strcpy(shows, gethomedir(NULL));
 
-		s = FileBrowse(shows, fname, NULL);
+		s = FileBrowse(shows, fname, NULL, FB_SAVE);
 		strcat(shows, S_FILESEP);
 		strcat(shows, fname);
 		strcpy(fname, shows);
@@ -383,7 +402,7 @@ int f, n;
 
 		sprintf(shows, "File \"%s\" exists, OVERWRITE", fname);
 		if((s=mlyesno(shows, FALSE)) != TRUE){
-		    if((bufp = strrchr(fname,'/')) == NULL)
+		    if((bufp = strrchr(fname,C_FILESEP)) == NULL)
 		      fname[0] = '\0';
 		    else
 		      *++bufp = '\0';
@@ -525,7 +544,10 @@ int f, n;
         if ((s=ffwopen(fn)) != FIOSUC)          /* Open writes message. */
                 return(NULL);
 
-	chmod(fn, 0600);			/* fix access rights */
+#ifndef _WINDOWS	/* For some reason, in windows changing the mode */
+			/* _always_ causes the file close to fail. */
+	chmod(fn, MODE_READONLY);		/* fix access rights */
+#endif
 
         lp = lforw(curbp->b_linep);             /* First line.          */
         nline = 0;                              /* Number of lines.     */
