@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: adrbklib.h,v 4.28 1994/08/22 23:33:47 hubert Exp $
+  $Id: adrbklib.h,v 4.33 1994/10/04 21:46:17 hubert Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -164,17 +164,41 @@
  * save processing all of the entries in between.
  */
 
-typedef unsigned short adrbk_cntr_t;  /* at most 2^16 entries         */
+/* #define HUGE_ADDRBOOKS */
+
+#ifdef HUGE_ADDRBOOKS
+
+/*
+ * Assume long >= 32 bits and use only 32 bits for portability of addrbook.
+ */
+typedef unsigned long adrbk_cntr_t;  /* addrbook counter type                */
+typedef unsigned long a_c_arg_t;     /* type of arg passed for adrbk_cntr_t  */
+#define MAX_ADRBK_SIZE (2000000000L) /* leave room for extra display lines   */
+#define MAX_HASHTABLE_SIZE 150000
+
+# else /* !HUGE_ADDRBOOKS */
+
+/*
+ * Assume short >= 16 bits and use only 16 bits for portability of addrbook.
+ */
+typedef unsigned short adrbk_cntr_t; /* addrbook counter type                */
+typedef unsigned a_c_arg_t;          /* type of arg passed for addrbk_cntr_t */
+#define MAX_ADRBK_SIZE ((long)(NO_NEXT - 2))
+#define MAX_HASHTABLE_SIZE 60000
+
+# endif /* !HUGE_ADDRBOOKS */
+
 /*
  * This value is reserved to mean that there is no next address, or that
  * there is no address number to return.  This is similar to getc returning
  * -1 when there is no char to get, but since we've defined this to be
  * unsigned we need to reserve one of the valid values for this purpose.
+ * With current implementation it needs to be all 1's, so memset initialization
+ * will work correctly.
  */
 #define NO_NEXT ((adrbk_cntr_t)-1)
-#define MAX_ADRBK_SIZE ((long)(NO_NEXT - 2))
 
-typedef long  adrbk_uid_t;   /* the UID of a name or address */
+typedef long adrbk_uid_t;   /* the UID of a name or address */
 
 typedef enum {ReadOnly, ReadWrite, NoAccess, NoExists} AccessType;
 typedef enum {NotSet, Single, List} Tag;
@@ -237,6 +261,7 @@ typedef struct width_stuff {
 } WIDTH_INFO_S;
 
 typedef struct adrbk {
+    char         *orig_filename;       /* passed in filename                 */
     char         *filename;            /* addrbook filename                  */
     char         *temp_filename;       /* tmp file while writing out changes */
     FILE         *fp;                  /* fp for filename                    */
@@ -247,7 +272,7 @@ typedef struct adrbk {
     char         *temp_hashfile;       /* tmp file while writing out changes */
     FILE         *fp_hash;
     AccessType    hashfile_access;     /* access permission for hashfile     */
-    int           htable_size;         /* how many entries in AdrHash tables */
+    adrbk_cntr_t  htable_size;         /* how many entries in AdrHash tables */
     adrbk_cntr_t  count;               /* how many entries in addrbook       */
     time_t        last_change_we_know_about;/* to look for others changing it*/
     ER_CACHE_ELEM_S **head_cache_elem; /* array of cache elem heads          */
@@ -308,7 +333,10 @@ typedef struct adrbk {
  *       Header -- magic number "P # * E @"
  *            <SPACE> two character version string
  *            <SPACE> hash table size "\n" integer padded on left with spaces
- *					so it takes up 5 character slots
+ * ifdef HUGE_ADDRBOOKS
+ *	It takes up 10 character slots in version 03
+ * else
+ *	Value restricted to take up 5 character slots in version 02
  *
  * EntryRef Arr -- An array of N OnDiskEntryRef's, where N is the number
  *                     of entries in the address book.  Each entry is followed
@@ -316,12 +344,14 @@ typedef struct adrbk {
  *
  *    HashTables -- Arrays of hash table size integers, each of which is
  *                     padded on left with spaces so it takes up 5 character
+ *                     (10 if HUGE_ADDRBOOKS)
  *                     slots.  Each integer ends with a "\n" to make it
  *                     easier to use debugging tools.
  *    and another one of those for by address hash table
  *
  *      Trailer -- magic number "P # * E @"
  *            <SPACE> N  "\n" the same N as the number of EntryRef entries
+ *        (N is 10 if HUGE_ADDRBOOKS, 5 otherwise)
  *           W1 W2 W3 W4 W5 "\n"   These W's are widths for helping
  *                             to make a nice display of the addrbook.  Each
  *                             consists of a SPACE followed by a one or two
@@ -339,6 +369,7 @@ typedef struct adrbk {
  * Each OnDiskEntryRef looks like:
  *      next_nick  -- 5 bytes, positive integer padded on left with spaces.
  *      next_addr  -- 5 bytes, positive integer padded on left with spaces.
+ *                   (each of those is 10 with HUGE_ADDRBOOKS)
  *      uid_nick   -- 11 bytes, integer padded on left with spaces.
  *      uid_addr   -- 11 bytes, integer padded on left with spaces.
  *      offset     -- 10 bytes, positive integer padded on left with spaces.
@@ -347,7 +378,11 @@ typedef struct adrbk {
  */
 
 #define ADRHASH_FILE_SUFFIX      ".lu"
-#define ADRHASH_FILE_VERSION_NUM "02"
+#ifdef HUGE_ADDRBOOKS
+#define ADRHASH_FILE_VERSION_NUM "04"
+#else
+#define ADRHASH_FILE_VERSION_NUM "03"
+#endif
 #define PMAGIC                   "P#*E@"
 #define LEGACY_PMAGIC            "P#*@ "  /* sorry about that */
 
@@ -360,28 +395,48 @@ typedef struct adrbk {
 #define SIZEOF_ASCII_USHORT (5)
 #define SIZEOF_ASCII_ULONG  (10)
 #define SIZEOF_ASCII_LONG   (11)  /* because of possible "-" sign */
+#define SIZEOF_UID          SIZEOF_ASCII_LONG
+#define SIZEOF_FILEOFFSET   SIZEOF_ASCII_ULONG
+#define SIZEOF_TIMESTAMP    SIZEOF_ASCII_ULONG
+
+#ifdef HUGE_ADDRBOOKS
+
+#define SIZEOF_HASH_SIZE    SIZEOF_ASCII_ULONG
+#define SIZEOF_HASH_INDEX   SIZEOF_ASCII_ULONG
+
+#else /* !HUGE_ADDRBOOKS */
+
+#define SIZEOF_HASH_SIZE    SIZEOF_ASCII_USHORT
+#define SIZEOF_HASH_INDEX   SIZEOF_ASCII_USHORT
+
+#endif /* !HUGE_ADDRBOOKS */
+
+#define SIZEOF_COUNT        SIZEOF_HASH_INDEX
+
 #define TO_FIND_HDR_PMAGIC  (0)
 #define TO_FIND_VERSION_NUM (TO_FIND_HDR_PMAGIC + SIZEOF_PMAGIC + SIZEOF_SPACE)
 #define TO_FIND_HTABLE_SIZE (TO_FIND_VERSION_NUM + \
 			     SIZEOF_VERSION_NUM + SIZEOF_SPACE)
 #define SIZEOF_HDR (TO_FIND_HTABLE_SIZE + \
-		    SIZEOF_ASCII_USHORT + SIZEOF_NEWLINE)
+		    SIZEOF_HASH_SIZE + SIZEOF_NEWLINE)
+
 #define TO_FIND_SORT_RULE (-(SIZEOF_SORT_RULE + SIZEOF_NEWLINE))
 #define TO_FIND_TIMESTAMP (TO_FIND_SORT_RULE + \
-                        (-(SIZEOF_ASCII_ULONG + SIZEOF_SPACE)))
+                        (-(SIZEOF_TIMESTAMP + SIZEOF_SPACE)))
 #define TO_FIND_WIDTHS (TO_FIND_TIMESTAMP + \
 		    (-(5 * (SIZEOF_SPACE + SIZEOF_WIDTH) + SIZEOF_NEWLINE)))
 #define TO_FIND_COUNT (TO_FIND_WIDTHS + \
-			(-(SIZEOF_ASCII_USHORT + SIZEOF_NEWLINE)))
+			(-(SIZEOF_COUNT + SIZEOF_NEWLINE)))
 #define TO_FIND_TRLR_PMAGIC (TO_FIND_COUNT + \
 			(-(SIZEOF_PMAGIC + SIZEOF_SPACE)))
 #define SIZEOF_TRLR (-(TO_FIND_TRLR_PMAGIC))
-#define SIZEOF_ENTRYREF_ENTRY (SIZEOF_ASCII_USHORT + SIZEOF_SPACE + \
-			       SIZEOF_ASCII_USHORT + SIZEOF_SPACE + \
-			       SIZEOF_ASCII_LONG   + SIZEOF_SPACE + \
-			       SIZEOF_ASCII_LONG   + SIZEOF_SPACE + \
-			       SIZEOF_ASCII_ULONG  + SIZEOF_SPACE)
-#define SIZEOF_HTABLE_ENTRY (SIZEOF_ASCII_USHORT + SIZEOF_SPACE)
+
+#define SIZEOF_ENTRYREF_ENTRY (SIZEOF_HASH_INDEX + SIZEOF_SPACE + \
+			       SIZEOF_HASH_INDEX + SIZEOF_SPACE + \
+			       SIZEOF_UID        + SIZEOF_SPACE + \
+			       SIZEOF_UID        + SIZEOF_SPACE + \
+			       SIZEOF_FILEOFFSET + SIZEOF_NEWLINE)
+#define SIZEOF_HTABLE_ENTRY (SIZEOF_HASH_INDEX + SIZEOF_NEWLINE)
 
 
 #ifdef ANSI
@@ -391,20 +446,20 @@ void            adrbk_clearrefs(AdrBk *);
 void            adrbk_close(AdrBk *);
 void            adrbk_partial_close(AdrBk *);
 adrbk_cntr_t    adrbk_count(AdrBk *);
-int             adrbk_delete(AdrBk *, unsigned);
+int             adrbk_delete(AdrBk *, a_c_arg_t);
 char           *adrbk_formatname(char *);
-AdrBk_Entry    *adrbk_get_ae(AdrBk *, unsigned, Handling);
+AdrBk_Entry    *adrbk_get_ae(AdrBk *, a_c_arg_t, Handling);
 int             adrbk_is_in_sort_order(AdrBk *);
-int             adrbk_listadd(AdrBk *, unsigned, char *);
-int             adrbk_nlistadd(AdrBk *, unsigned, char **);
-int             adrbk_listdel(AdrBk *, unsigned, char *);
+int             adrbk_listadd(AdrBk *, a_c_arg_t, char *);
+int             adrbk_nlistadd(AdrBk *, a_c_arg_t, char **);
+int             adrbk_listdel(AdrBk *, a_c_arg_t, char *);
 AdrBk_Entry    *adrbk_lookup_by_addr(AdrBk *, char *, adrbk_cntr_t *);
 AdrBk_Entry    *adrbk_lookup_by_nick(AdrBk *, char *, adrbk_cntr_t *);
-AdrBk_Entry    *adrbk_newentry();
-AdrBk          *adrbk_open(char *, char *, char *, int, int);
+AdrBk_Entry    *adrbk_newentry(void);
+AdrBk          *adrbk_open(char *, char *, char *, int, int, int);
 long            adrbk_get_nominal_cachesize(AdrBk *);
 long            adrbk_set_nominal_cachesize(AdrBk *, long);
-int             adrbk_sort(AdrBk *, unsigned, adrbk_cntr_t *, int);
+int             adrbk_sort(AdrBk *, a_c_arg_t, adrbk_cntr_t *, int);
 
 #else /* !ANSI */
 
