@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: filter.c,v 4.29 1994/08/18 17:39:51 mikes Exp $";
+static char rcsid[] = "$Id: filter.c,v 4.30 1994/09/30 21:59:46 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -759,6 +759,9 @@ gf_pipe(gc, pc)
     StartInverse();
     n = 0L;
 #endif
+
+    dprint(4, (debugfile, "-- gf_pipe: "));
+
     /*
      * set up for any errors a filter may encounter
      */
@@ -767,6 +770,8 @@ gf_pipe(gc, pc)
 	ibmoutc(' ');
 	EndInverse();
 #endif
+	dprint(4, (debugfile, "ERROR: %s\n",
+		   gf_error_string ? gf_error_string : "NULL"));
 	return(gf_error_string); 	/*  */
     }
 
@@ -798,6 +803,7 @@ gf_pipe(gc, pc)
     ibmoutc(' ');
     EndInverse();
 #endif
+    dprint(1, (debugfile, "done.\n"));
     return(NULL);			/* everything went OK */
 }
 
@@ -927,6 +933,7 @@ gf_binary_b64(f, c, flg)
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
     else if(flg == GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset binary_b64\n"));
 	f->n = 0L;
     }
 }
@@ -999,10 +1006,10 @@ gf_b64_binary(f, c, flg)
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
     else if(flg == GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset b64_binary\n"));
 	f->n  = 0L;			/* quantum position */
 	f->f1 = 0;			/* state holder: equal seen? */
     }
-
 }
 
 
@@ -1108,10 +1115,10 @@ gf_qp_8bit(f, c, flg)
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
     else if(flg == GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset qp_8bit\n"));
 	f->f1 = DFL;
 	f->linep = f->line = (char *)fs_get(GF_MAXLINE * sizeof(char));
     }
-
 }
 
 
@@ -1194,6 +1201,7 @@ gf_8bit_qp(f, c, flg)
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
     else if(flg == GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset 8bit_qp\n"));
 	f->f1 = DFL;			/* state from last character        */
 	f->f2 = 1;			/* state of "^NFrom " bitmap        */
 	f->t  = 0;
@@ -1395,11 +1403,11 @@ gf_rich2plain(f, c, flg)
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
     else if(flg == GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset rich2plain\n"));
 	f->f1 = DFL;			/* state */
 	f->f2 = 0;			/* set means we're in a comment */
 	f->linep = f->line = (char *)fs_get(45 * sizeof(char));
     }
-
 }
 
 
@@ -1553,11 +1561,11 @@ gf_enriched2plain(f, c, flg)
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
     else if(flg == GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset enriched2plain\n"));
 	f->f1 = DFL;			/* state */
 	f->f2 = 0;			/* set means we're in a comment */
 	f->linep = f->line = (char *)fs_get(65 * sizeof(char));
     }
-
 }
 
 
@@ -1673,11 +1681,11 @@ gf_wrap(f, c, flg)
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
     else if(flg == GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset wrap\n"));
 	f->f1    = DFL;
 	f->n     = 0L;
 	f->linep = f->line = (char *)fs_get((gf_wrap_width+10)*sizeof(char));
     }
-
 }
 
 
@@ -1754,10 +1762,10 @@ gf_prefix(f, c, flg)
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
     else if(flg == GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset prefix\n"));
 	f->f1 = DFL;
 	f->n  = 0L;
     }
-
 }
 
 
@@ -1815,6 +1823,7 @@ gf_nvtnl_local(f, c, flg)
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
     else if(flg == GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset nvtnl_local\n"));
 	f->f1 = DFL;
     }
 }
@@ -1848,6 +1857,11 @@ gf_local_nvtnl(f, c, flg)
     else if(flg == GF_EOD){
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
+    else if(GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset local_nvtnl\n"));
+	/* no op */
+    }
+
 }
 
 #ifdef	DOS
@@ -1870,7 +1884,6 @@ gf_translate(f, c, flg)
     FILTER_S *f;
     int       c, flg;
 {
-    /* GF_RESET case a no-op */
     if(flg == GF_DATA){
 	if(c >= 0 && c < (int) gf_xlate_tab_len)
 	  (*f->next->f)(f->next, (int)gf_xlate_tab[c], GF_DATA);
@@ -1878,6 +1891,10 @@ gf_translate(f, c, flg)
     else if(flg == GF_EOD){
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
+    else if(GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset translate\n"));
+	/* no op */
+    }
 }
 
 
@@ -1926,6 +1943,7 @@ gf_busy(f, c, flg)
 	(*f->next->f)(f->next, 0, GF_EOD);
     }
     else if(flg == GF_RESET){
+	dprint(9, (debugfile, "-- gf_reset busy\n"));
 	f->f1 = 0;
         x = 0;
 	StartInverse();
