@@ -56,9 +56,12 @@ static char rcsid[] = "$Id: pine-use.c,v 4.4 1994/02/25 18:17:20 hubert Exp $";
 #include <sys/stat.h>
 
 #ifndef MAILSPOOLPCTS
-#define MAILSPOOLPCTS "/usr/spool/mail/%s"
-/* #define MAILSPOOLPCTS "/usr/mail/%s" */
-#endif
+#ifdef __linux__
+#define MAILSPOOLPCTS "/var/spool/mail/%s"
+#else  /* __linux__ */
+#define MAILSPOOLPCTS "/usr/mail/%s"
+#endif /* __linux__ */
+#endif /* MAILSPOOLPCTS */
 
 #define DAYSEC (60*60*24)
 
