@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: ttyin.c,v 4.52 1994/08/18 17:40:15 mikes Exp $";
+static char rcsid[] = "$Id: ttyin.c,v 4.56 1994/10/11 22:53:43 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -187,7 +187,9 @@ int state;
 	_raw_tty.c_lflag &= ~(ICANON | ECHO | IEXTEN); /* noecho raw mode    */
 
  	_raw_tty.c_lflag &= ~ISIG;		/* disable signals           */
- 	_raw_tty.c_iflag &= ~IXON;		/* turn off ^S/^Q on input   */
+	if(F_OFF(F_PRESERVE_START_STOP, ps_global))
+	  _raw_tty.c_iflag &= ~IXON;		/* turn off ^S/^Q on input   */
+
  	_raw_tty.c_iflag &= ~ICRNL;		/* turn off CR->NL on input  */
  	_raw_tty.c_oflag &= ~ONLCR;		/* turn off NL->CR on output */
  	_raw_tty.c_oflag &= ~OPOST;
@@ -215,7 +217,9 @@ int state;
 	_raw_tty.c_lflag &= ~(ICANON | ECHO);	/* noecho raw mode  */
 
  	_raw_tty.c_lflag &= ~ISIG;		/* disable signals */
- 	_raw_tty.c_iflag &= ~IXON;		/* turn off ^S/^Q on input   */
+	if(F_OFF(F_PRESERVE_START_STOP, ps_global))
+	  _raw_tty.c_iflag &= ~IXON;		/* turn off ^S/^Q on input   */
+
  	_raw_tty.c_iflag &= ~ICRNL;		/* turn off CR->NL on input  */
  	_raw_tty.c_oflag &= ~ONLCR;		/* turn off NL->CR on output */
  	_raw_tty.c_oflag &= ~OPOST;
@@ -242,8 +246,10 @@ int state;
 	_raw_tchars.t_intrc = -1;	/* Turn off ^C and ^D */
 	_raw_tchars.t_eofc  = -1;
 
-	_raw_tchars.t_startc = -1;	/* Turn off ^S/^Q */
-	_raw_tchars.t_stopc  = -1;
+	if(F_OFF(F_PRESERVE_START_STOP, ps_global)){
+	    _raw_tchars.t_startc = -1;	/* Turn off ^S/^Q */
+	    _raw_tchars.t_stopc  = -1;
+	}
 
 #ifdef	DEBUG
 	if(debug < 9)			/* only on if full debugging set */
@@ -307,7 +313,8 @@ read_with_timeout(time_out)
      int    res;
 #else
      struct timeval tmo;
-     int            readfds, res;
+     fd_set         readfds, errfds;
+     int	    res;
 #endif
      unsigned char  c;
 
@@ -334,25 +341,25 @@ read_with_timeout(time_out)
      if(time_out > 0) {
          /* Check to see if there's bytes to read with a timeout */
 #ifdef USE_POLL
-	pollfd.fd = STDIN_FD;
-	pollfd.events = POLLIN;
-	dprint(9,(debugfile,"poll event %d, timeout %d\n", pollfd.events,
-		  time_out));
-	res = poll (&pollfd, 1, time_out * 1000);
-	dprint(9, (debugfile, "poll on tty returned %d, events %d\n",
-		   res, pollfd.revents));
+	 pollfd.fd = STDIN_FD;
+	 pollfd.events = POLLIN;
+	 dprint(9,(debugfile,"poll event %d, timeout %d\n", pollfd.events,
+		   time_out));
+	 res = poll (&pollfd, 1, time_out * 1000);
+	 dprint(9, (debugfile, "poll on tty returned %d, events %d\n",
+		    res, pollfd.revents));
 #else
-         if(STDIN_FD == 0)
-           readfds    = 1;
-         else
-           readfds    = 1 << STDIN_FD; /* First bit for file desc 0 */
-         tmo.tv_sec  = time_out;
+	 FD_ZERO(&readfds);
+	 FD_ZERO(&errfds);
+	 FD_SET(STDIN_FD, &readfds);
+	 FD_SET(STDIN_FD, &errfds);
+	 tmo.tv_sec  = time_out;
          tmo.tv_usec = 0; 
 
          dprint(9, (debugfile,"Select readfds:%d timeval:%d,%d\n", readfds,
 		   tmo.tv_sec,tmo.tv_usec));
 
-         res = select(STDIN_FD+1, &readfds, 0, &readfds, &tmo);
+         res = select(STDIN_FD+1, &readfds, 0, &errfds, &tmo);
 
          dprint(9, (debugfile, "Select on tty returned %d\n", res));
 #endif
@@ -1513,6 +1520,13 @@ optionally_enter(string, y_base, x_base, field_len, append_current, passwd,
             break;
 
 
+	    /*-------------------- Help --------------------*/
+	  case ctrl('G') : 
+	  case PF1:
+	    return_v = 3;
+	    break;
+
+
           case NO_OP_COMMAND:
           case NO_OP_IDLE:
             new_mail(&cursor_moved, 0, 2); /* Keep mail stream alive */
@@ -1520,13 +1534,6 @@ optionally_enter(string, y_base, x_base, field_len, append_current, passwd,
               break;
             /* Else fall into redraw */
 
-	    
-	    /*-------------------- Help --------------------*/
-	  case ctrl('G') : 
-	  case PF1:
-	    return_v = 3;
-	    break;
-
 	    /*-------------------- Redraw --------------------*/
 	  case ctrl('L'):
             /*---------------- re size ----------------*/
